
        .XCREF
        .NOLIST
        INCLUDE    \masm32\include\masm32rt.inc
        .LIST

;#############################################################################################################

TABLE_SIZE EQU 1024       ;1024 filename table entries
NAMES_SIZE EQU 262144     ;256 Kb filename buffer

;#############################################################################################################

AddFile PROTO :DWORD

;#############################################################################################################

        .DATA

szMatch db '*.*',0
szCrLf  db 13,10,0

;*************************************************************************************************************

        .DATA?

hHeap   dd ?
lpBlock dd ?
lpNextE dd ?
lpNames dd ?
lpNextN dd ?
lpEnd   dd ?
hFind   dd ?

wfd WIN32_FIND_DATA <>
; dwFileAttributes    dd ?
; ftCreationTime      FILETIME <>
; ftLastAccessTime    FILETIME <>
; ftLastWriteTime     FILETIME <>
; nFileSizeHigh       dd ?
; nFileSizeLow        dd ?
; dwReserved0         dd ?
; dwReserved1         dd ?
; cFileName           db MAX_PATH dup(?) ;MAX_PATH = 260
; cAlternate          db 14 dup(?)

;#############################################################################################################

        .CODE

_main   PROC

;-----------------------------------------------------

;initialize hHeap (process heap handle)

        INVOKE  GetProcessHeap
        mov     hHeap,eax

;-----------------------------------------------------

;allocate the buffer
;we use one large buffer to hold both the table and the file names
;each table entry is an address to a file name string

TOTAL_SIZE = 4*TABLE_SIZE+NAMES_SIZE

        INVOKE  HeapAlloc,hHeap,NULL,TOTAL_SIZE

;lpBlock is the address of the allocated heap block
;we also use that address for our table and the initial address of the "next entry"

        mov     lpBlock,eax
        mov     lpNextE,eax

;we add the size of the table to that address to calculate the name buffer address
;which is also the initial address of the "next name"

        add     eax,4*TABLE_SIZE
        mov     lpNames,eax
        mov     lpNextN,eax

;it simplifies our code if we know where the end of our name buffer is

        add     eax,NAMES_SIZE
        mov     lpEnd,eax

;-----------------------------------------------------

;fill the table/name buffer with file names
;if we are out of files, table space, or name space, we stop

        INVOKE  FindFirstFile,offset szMatch,offset wfd
        cmp     eax,INVALID_HANDLE_VALUE
        jz      Find02

        mov     hFind,eax

Find00: INVOKE  AddFile,offset wfd.cFileName
        or      eax,eax
        jnz     Find01

        INVOKE  FindNextFile,hFind,offset wfd
        or      eax,eax
        jnz     Find00

Find01: INVOKE  FindClose,hFind

Find02:

;-----------------------------------------------------

;display the names from the buffer

        push    esi
        mov     esi,lpBlock       ;ESI points to the first table entry
        jmp short Show01

Show00: mov     eax,[esi]         ;get the name string address from the table
        print   eax               ;display the file name
        print   offset szCrLf     ;display a carriage return/line feed
        add     esi,4             ;point to the next table entry

Show01: cmp     esi,lpNextE       ;if it points to where the "next entry" goes, we are done
        jb      Show00

        pop     esi

;-----------------------------------------------------

;free the allocated block

        INVOKE  HeapFree,hHeap,NULL,lpBlock

;-----------------------------------------------------

;wait for a keypress and exit

        inkey
        exit

_main   ENDP

;*************************************************************************************************************

AddFile PROC USES EBX ESI EDI lpFileName:DWORD
;
;Add File to Buffer
;
;Call With: lpFileName = address of zero-terminated file name string
;
;  Returns: EAX = status:
;                 0 if file name was added to buffer
;                 1 if table is full
;                 2 if name buffer is full
;
;Also Uses: ECX, EDX, all other registers are preserved
;
;-----------------------------------------------------

        mov     ebx,lpNextE
        mov     eax,1
        cmp     ebx,lpNames
        jae     AddF00             ;exit with EAX=1 if table is full

        mov     esi,lpFileName
        INVOKE  StrLen,esi
        inc     eax                ;add 1 to string length for terminator
        mov     edi,lpNextN
        mov     edx,eax
        mov     ecx,eax
        add     edx,edi
        mov     eax,2
        cmp     edx,lpEnd
        jae     AddF00             ;exit with EAX=2 if name won't fit in buffer

        mov     [ebx],edi          ;update the table entry
        rep     movsb              ;copy the file name into the name buffer
        add     ebx,4
        mov     lpNextN,edi        ;update the "next name" address
        mov     lpNextE,ebx        ;update the "next entry" address
        xor     eax,eax            ;exit with EAX=0 if name was added

AddF00: ret

AddFile ENDP

;#############################################################################################################

        END     _main
